
//+-------------------------------------------------------------------------+
//| BullsBearsEyes.mq4                                                      |
//| EmeraldKing , transport_david                                           |
//| http://finance.groups.yahoo.com/group/MetaTrader_Experts_and_Indicators/|
//+-------------------------------------------------------------------------+
#property copyright "Copyright  2005"
#property link      "http://finance.groups.yahoo.com/group/MetaTrader_Experts_and_Indicators/"

#property indicator_separate_window
#property indicator_buffers 3
#property indicator_color1  clrOrange
#property indicator_color2  clrLime
#property indicator_color3  clrRed
#property indicator_minimum -0.05
#property indicator_maximum 1.05

//
//
//
//
//

extern double             periods       = 13;
extern ENUM_MA_METHOD     MaMethod      = MODE_LWMA;
extern ENUM_APPLIED_PRICE MaPrice       = PRICE_CLOSE;
extern double             gamma         = 0.6;
extern double             Level1        = 0.75;
extern double             Level2        = 0.45;
extern double             Level3        = 0.15;
extern bool               ShowLevels    = true; 
extern bool               ShowCrossings = true; 
extern bool               alertsOn      = false;
extern bool               alertsMessage = true;
extern bool               alertsSound   = false;
extern bool               alertsEmail   = false;
extern color              LevelsColor   = C'30,33,36';
extern int                DotSize       = 3;

double lag[];
double Up[];
double Dn[];
double L0[];
double L1[];
double L2[];
double L3[];
double bears[];
double bulls[];
string ShortName;

//
//
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+

int init()
{
   IndicatorBuffers(9);
   SetIndexBuffer(0,lag); SetIndexLabel(0,"BullBearEyes");
   SetIndexBuffer(1,Up);
   SetIndexBuffer(2,Dn);
   SetIndexBuffer(3,L0);
   SetIndexBuffer(4,L1);
   SetIndexBuffer(5,L2);
   SetIndexBuffer(6,L3);
   SetIndexBuffer(7,bears);
   SetIndexBuffer(8,bulls);
   
   //
   //
   //
   //
   //
   
   if (ShowCrossings)
   {
      SetIndexStyle(1,DRAW_ARROW,0,DotSize); SetIndexArrow(1,159);         
      SetIndexStyle(2,DRAW_ARROW,0,DotSize); SetIndexArrow(2,159);                     
   }
   else
   {
      SetIndexStyle(1,DRAW_NONE);
      SetIndexStyle(2,DRAW_NONE);
   }      

   ShortName=MakeUniqueName(" BullBearEyes "," ("+DoubleToStr(gamma,2)+")");
   IndicatorShortName(ShortName);
   return(0);
}
int deinit()
{
   DeleteBounds();
   return(0);
}

//
//
//
//
//

int start()
{
   int i,counted_bars=IndicatorCounted();
      if(counted_bars < 0) return(-1);
      if(counted_bars>0) counted_bars--;
         int limit = MathMin(Bars-counted_bars,Bars-1);
      
      //
      //
      //
      //
      //
      
      for(i=limit; i>=0; i--) bears[i] = Low[i]  - iMA(NULL,0,periods,0,MaMethod,MaPrice,i);
      for(i=limit; i>=0; i--) bulls[i] = High[i] - iMA(NULL,0,periods,0,MaMethod,MaPrice,i);
      for(i=limit; i>=0; i--)
      {
         double prc = bulls[i]+bears[i];
	      double RSI = 0.00;
	      double CU  = 0.00;
	      double CD  = 0.00;
	   
	      //
	      //
	      //
	      //
	      //
	   
         L0[i] = (1.0 - gamma)* prc     + gamma*L0[i+1];
	      L1[i] = -gamma*L0[i] + L0[i+1] + gamma*L1[i+1];
	      L2[i] = -gamma*L1[i] + L1[i+1] + gamma*L2[i+1];
	      L3[i] = -gamma*L2[i] + L2[i+1] + gamma*L3[i+1];

	      if (L0[i] >= L1[i]) CU = L0[i] - L1[i];
	      else	              CD = L1[i] - L0[i];
	      if (L1[i] >= L2[i]) CU = CU + L1[i] - L2[i];
	      else	              CD = CD + L2[i] - L1[i];
	      if (L2[i] >= L3[i]) CU = CU + L2[i] - L3[i];
	      else	              CD = CD + L3[i] - L2[i];
	      if (CU + CD != 0)   lag[i] = CU / (CU + CD);
	      
	      Up[i]  = EMPTY_VALUE;
         Dn[i]  = EMPTY_VALUE;
               if (lag[i] > Level1 && lag[i+1] < Level1) Up[i] = Level1;
               if (lag[i] < Level1 && lag[i+1] > Level1) Dn[i] = Level1;
               if (lag[i] > Level3 && lag[i+1] < Level3) Up[i] = Level3;
               if (lag[i] < Level3 && lag[i+1] > Level3) Dn[i] = Level3;
	}
	if (ShowLevels) UpdateBounds();
   if (alertsOn)   CheckCrossings();
return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

void CheckCrossings()
{
   if (Up[0] == Level1) doAlert("level "+DoubleToStr(Level1,2)+" crossed up");
   if (Dn[0] == Level1) doAlert("level "+DoubleToStr(Level1,2)+" crossed down");
   if (Up[0] == Level3) doAlert("level "+DoubleToStr(Level3,2)+" crossed up");
   if (Dn[0] == Level3) doAlert("level "+DoubleToStr(Level3,2)+" crossed down");
}

//
//
//
//
//

void doAlert(string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[0]) {
          previousAlert  = doWhat;
          previousTime   = Time[0];

          //
          //
          //
          //
          //

          message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," BullBearsEyes ",doWhat);
             if (alertsMessage) Alert(message);
             if (alertsEmail)   SendMail(StringConcatenate(Symbol()," BullBearsEyes line crossing"),message);
             if (alertsSound)   PlaySound("alert2.wav");
      }
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

string MakeUniqueName(string first, string rest)
{
   string result = first+(MathRand()%1001)+rest;

   while (WindowFind(result)> 0)
          result = first+(MathRand()%1001)+rest;
   return(result);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

void DeleteBounds()
{
   ObjectDelete(ShortName+"-1");
   ObjectDelete(ShortName+"-2");
   ObjectDelete(ShortName+"-3");
}

//
//
//
//
//

void UpdateBounds()
{
   if (Level1 > 0) SetUpBound(ShortName+"-1",1.0000     ,Level1);
   if (Level2 > 0) SetUpBound(ShortName+"-2",Level2*1.01,Level2*0.99);
   if (Level3 > 0) SetUpBound(ShortName+"-3",Level3     ,     0.0000);
}

//
//
//
//
//

void SetUpBound(string name, double up, double down,int objType=OBJ_RECTANGLE)
{
   if (ObjectFind(name) == -1)
      {
         ObjectCreate(name,objType,WindowFind(ShortName),0,0);
         ObjectSet(name,OBJPROP_PRICE1,up);
         ObjectSet(name,OBJPROP_PRICE2,down);
         ObjectSet(name,OBJPROP_COLOR,LevelsColor);
         ObjectSet(name,OBJPROP_BACK,true);
         ObjectSet(name,OBJPROP_TIME1,iTime(NULL,0,Bars-1));
      }
   if (ObjectGet(name,OBJPROP_TIME2) != iTime(NULL,0,0))
       ObjectSet(name,OBJPROP_TIME2,    iTime(NULL,0,0));
}

